package com.das.common.interceptor;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletRequestWrapper;

import java.util.Collections;
import java.util.Enumeration;

public class TokenUpdatingHttpServletRequestWrapper extends HttpServletRequestWrapper {

    private final String newTokenValue;

    public TokenUpdatingHttpServletRequestWrapper(HttpServletRequest request, String newTokenValue) {
        super(request);
        this.newTokenValue = newTokenValue;
    }

    @Override
    public String getHeader(String name) {
        if ("token".equalsIgnoreCase(name)) {
            return newTokenValue; // 返回新的token值
        }
        return super.getHeader(name); // 对于其他header，委托给父类处理
    }

    @Override
    public Enumeration<String> getHeaders(String name) {
        if ("token".equalsIgnoreCase(name)) {
            return Collections.enumeration(Collections.singletonList(newTokenValue)); // 返回包含新token值的枚举
        }
        return super.getHeaders(name); // 对于其他header，委托给父类处理
    }

}
