package com.das.modules.auth.controller;

import cn.hutool.core.util.IdUtil;
import com.das.common.captcha.CaptchaProperties;
import com.das.common.captcha.CaptchaUtil;
import com.das.common.captcha.CaptchaVO;
import com.das.common.result.R;
import com.das.common.utils.AdminRedisTemplate;
import com.das.modules.auth.domain.LoginUserDetails;
import com.das.modules.auth.domain.request.LoginRequest;
import com.das.modules.auth.domain.request.RefreshTokenRequest;
import com.das.modules.auth.domain.vo.LoginVO;
import com.das.modules.auth.service.ILoginService;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.google.code.kaptcha.Producer;
import io.micrometer.common.util.StringUtils;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.NotEmpty;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;


/**
 * @author chenhaojie
 * @Description 登录控制层
 */
@Slf4j
@RequestMapping("/api/auth")
@RestController
public class LoginController {
    @Resource
    private ILoginService loginService;

    @Resource
    private Producer producer;
    @Resource
    private CaptchaProperties captchaProperties;

    @Resource
    private AdminRedisTemplate adminRedisTemplate;

    /**
     * 登录接口
     * @return 退出结果提示信息
     */
    @PostMapping("/login")
    public R<LoginUserDetails> login(@RequestBody LoginRequest loginRequest, HttpServletRequest request, HttpServletResponse response) throws JsonProcessingException {
        LoginUserDetails loginInfo = loginService.login(loginRequest,request, response);
        if (StringUtils.isNotEmpty(loginInfo.getMsg())){
            return R.fail(loginInfo.getMsg());
        }
        return R.success(loginInfo);

    }

    /**
     * 退出登录接口
     * @return 退出结果提示信息
     */
    @PostMapping("/logout")
    public R<?> logout(HttpServletRequest request) {
        if (loginService.logout(request)) {
            return R.success("退出成功");
        } else {
            return R.fail("退出失败");
        }

    }

    @PostMapping("/captchaImage")
    public R<CaptchaVO> getCaptcha() {
        String uuid = IdUtil.fastSimpleUUID();
        String imageBase64Str = CaptchaUtil.getImageBase64Str(producer, adminRedisTemplate, captchaProperties, uuid);
        CaptchaVO captchaVO = new CaptchaVO(uuid, imageBase64Str);
        return R.success(captchaVO);
    }

    @PostMapping("/refreshToken")
    public R<LoginVO> refreshToken(@RequestBody RefreshTokenRequest refreshToken){
        LoginVO loginVO  = loginService.refreshToken(refreshToken);
        return R.success(loginVO);
    }


}
