package com.das.modules.auth.service.impl;

import cn.dev33.satoken.secure.BCrypt;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.das.common.captcha.CaptchaUtil;
import com.das.common.config.SaTokenProperties;
import com.das.common.utils.AESUtil;
import com.das.common.utils.AdminRedisTemplate;
import com.das.modules.auth.domain.LoginUserDetails;
import com.das.modules.auth.domain.request.LoginRequest;
import com.das.modules.auth.domain.request.RefreshTokenRequest;
import com.das.modules.auth.domain.vo.LoginVO;
import com.das.modules.auth.domain.vo.SysUserVo;
import com.das.modules.auth.entity.SysUser;
import com.das.modules.auth.mapper.SysUserMapper;
import com.das.modules.auth.service.ILoginService;
import com.fasterxml.jackson.core.JsonProcessingException;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.Duration;

/**
 * @author chenhaojie
 * @Description
 */
@Service
public class LoginServiceImpl implements ILoginService {

    @Resource
    private SaTokenProperties saTokenProperties;
    @Resource
    private SysUserMapper sysUserMapper;

    @Autowired
    private AdminRedisTemplate adminRedisTemplate;

    @Override
    public LoginUserDetails login(LoginRequest loginRequest, HttpServletRequest request, HttpServletResponse response) throws JsonProcessingException {
        String name = loginRequest.getUsername();
        String password = loginRequest.getPassword();
        String code = loginRequest.getCode();
        String key = loginRequest.getKey();

        LoginUserDetails loginInfo = new LoginUserDetails();
        LambdaQueryWrapper<SysUser> wrapper = Wrappers.lambdaQuery();
        wrapper.eq(SysUser::getAccount, name);
        SysUser sysUser = sysUserMapper.selectOne(wrapper);
        if (sysUser == null) {
            loginInfo.setMsg("无账号信息");
            return loginInfo;
        }
        if (!CaptchaUtil.checkVerificationCode(key, code, adminRedisTemplate)) {
            loginInfo.setMsg("验证码不正确");
            return loginInfo;
        }
        if (!BCrypt.checkpw(password, sysUser.getPassword())) {
            loginInfo.setMsg("账号密码错误");
            return loginInfo;
        }
        StpUtil.login(sysUser.getAccount());// 执行登录，这里username为用户唯一标识
        String refreshTokenUuid = IdUtil.fastSimpleUUID();
        String token = StpUtil.getTokenValue().replace("-", "");
        String refreshToken = "refresh:" + refreshTokenUuid;

        SysUserVo sysUserVo = new SysUserVo();
        sysUserVo.setId(sysUser.getId());
        sysUserVo.setAccount(sysUser.getAccount());
        sysUserVo.setUserName(sysUser.getUserName());
        sysUserVo.setPhone(sysUser.getPhone());
        sysUserVo.setEmail(sysUser.getEmail());
        sysUserVo.setOrgId(sysUser.getOrgId());

        loginInfo.setSysUser(sysUserVo); // 存储用户信息到会话
        loginInfo.setToken(token);
        loginInfo.setRefreshToken(refreshTokenUuid);
        adminRedisTemplate.setEx(token, loginInfo, Duration.ofSeconds(saTokenProperties.getExpireTime()));
        adminRedisTemplate.setEx(refreshToken, loginInfo, Duration.ofSeconds(saTokenProperties.getRefreshExpireTime()));
        return loginInfo;
    }


    @Override
    public boolean logout(HttpServletRequest request) {
        String token = request.getHeader("token");
        adminRedisTemplate.del(token);
        StpUtil.logoutByTokenValue(token);
        return true;
    }

    @Override
    public LoginVO refreshToken(RefreshTokenRequest refreshToken) {
        LoginUserDetails loginUserDetails = adminRedisTemplate.get(refreshToken.getRefreshToken());
        LoginVO loginInfo = new LoginVO();
        if (loginUserDetails == null) {
            loginInfo.setMsg("token过期，请重新登录");
            return loginInfo;
        }
        StpUtil.login(loginUserDetails.getSysUser().getAccount());// 执行登录，这里username为用户唯一标识
        String newToken = StpUtil.getTokenValue().replace("-", "");;
        String newRefreshTokenUuid = IdUtil.fastSimpleUUID();
        String newRefreshToken = "refresh:" + newRefreshTokenUuid;
        loginInfo.setAccessToken(newToken);

        loginUserDetails.setToken(newToken);
        loginUserDetails.setRefreshToken(newRefreshTokenUuid);
        String oldRefreshToken = loginUserDetails.getRefreshToken();

        adminRedisTemplate.setEx(newToken, loginUserDetails, Duration.ofSeconds(saTokenProperties.getExpireTime()));
        adminRedisTemplate.setEx(newRefreshToken, loginUserDetails, Duration.ofSeconds(saTokenProperties.getRefreshExpireTime()));
        // 删除原有刷新token
        adminRedisTemplate.del("refresh:" + oldRefreshToken);

        return new LoginVO(newToken, newRefreshToken, saTokenProperties.getRefreshExpireTime());

    }

}
